from pyPhases import PluginAdapter
from pyPhasesRecordloader import RecordLoader


class Plugin(PluginAdapter):
    def initPlugin(self):
        # self.project.loadConfig(self.project.loadConfig(pathlib.Path(__file__).parent.absolute().joinpath("config.yaml")))
        RecordLoader.registerRecordLoader("RecordLoaderSHHS", "pyPhasesRecordloaderSHHS.recordLoaders")
        RecordLoader.registerRecordLoader("SHHSAnnotationLoader", "pyPhasesRecordloaderSHHS.recordLoaders")
        shhsPath = self.getConfig("shhs-path")
        self.project.setConfig("loader.shhs.dataset.downloader.basePath", shhsPath + "/polysomnography/edfs/shhs1")
        self.project.setConfig(
            "loader.shhs.dataset.downloader.basePathExtensionwise",
            [shhsPath + "polysomnography/edfs/shhs1", shhsPath + "polysomnography/annotations-events-nsrr/shhs1"],
        )

        self.project.setConfig("loader.shhs2.dataset.downloader.basePath", shhsPath + "/polysomnography/edfs/shhs2")
        self.project.setConfig(
            "loader.shhs2.dataset.downloader.basePathExtensionwise",
            [shhsPath + "polysomnography/edfs/shhs2", shhsPath + "polysomnography/annotations-events-nsrr/shhs2"],
        )



from pathlib import Path

# from .recordManager.RecordMeta import Record
from .SHHSAnnotationLoader import SHHSAnnotationLoader
from pyPhasesRecordloader.recordLoaders.EDFRecordLoader import EDFRecordLoader


class RecordLoaderSHHS(EDFRecordLoader):
    def __init__(
        self,
        filePath,
        targetSignals,
        targetSignalTypes=[],
        optionalSignals=[],
        combineChannels=[],
    ) -> None:
        super().__init__(
            filePath,
            targetSignals,
            targetSignalTypes=targetSignalTypes,
            optionalSignals=optionalSignals,
            combineChannels=combineChannels,
        )

        self.exportsEventArray = True

    def getFileBasePath(self, recrdId):
        return self.filePath

    def getFilePathSignal(self, recordId):
        return self.getFileBasePath(recordId) + "/" + recordId + ".edf"

    def getFilePathAnnotation(self, recordId):
        return self.getFileBasePath(recordId) + "/" + recordId + "-nsrr.xml"

    def existAnnotation(self, recordId):
        return Path(self.getFilePathAnnotation(recordId)).exists()

    def exist(self, recordId):
        return (
            Path(self.getFilePathAnnotation(recordId)).exists()
            & Path(self.getFilePathSignal(recordId)).exists()
        )

    def loadAnnotation(self, recordId, fileName, valueMap=None):
        filePath = self.getFilePathAnnotation(recordId)
        annotationLoader = SHHSAnnotationLoader.load(
            filePath, valueMap, self.annotationFrequency
        )

        return annotationLoader.events

    def getEventList(self, recordName, targetFrequency=1):
        metaXML = self.getFilePathAnnotation(recordName)
        xmlLoader = SHHSAnnotationLoader()

        eventArray = xmlLoader.loadAnnotation(metaXML)
        self.lightOff = xmlLoader.lightOff
        self.lightOn = xmlLoader.lightOn

        if targetFrequency != 1:
            eventArray = self.updateFrequencyForEventList(eventArray, targetFrequency)

        return eventArray

    # def getEventArray(self, recordName, signalLengths, annotationFrequency, forceGapBetweenEvents=False):
    #     events = self.getEventList(recordName)
    #     em = PSGEventManager()
    #     eventsDF = em.getDataframeFromEvents(events)

    #     return em.getEventSignalFromDF(
    #         eventsDF,
    #         signalLengths,
    #         annotationFrequency,
    #         forceGapBetweenEvents=forceGapBetweenEvents,
    #     )

    # def getMetaData(self, recordName) -> Record:

    #     self.frequency = 0

    #     edfFile = self.getFilePathSignal(recordName)

    #     record = Record()
    #     self.fillRecordFromEdf(record, edfFile)

    #     return record



from typing import List

from pyPhases.util.Logger import classLogger
from pyPhasesRecordloader import AnnotationInvalid, AnnotationNotFound, Event
from pyPhasesRecordloader.recordLoaders.XMLAnnotationLoader import XMLAnnotationLoader


@classLogger
class SHHSAnnotationLoader(XMLAnnotationLoader):
    stageMap = {
        "Wake|0": "W",
        "Stage 1 sleep|1": "N1",
        "Stage 2 sleep|2": "N2",
        "Stage 3 sleep|3": "N3",
        "Stage 4 sleep|4": "N3",
        "REM sleep|5": "R",
        # "Unsure|Unsure": "undefined",
        "Unscored|9": "undefined",
    }

    eventMap = {
        # "arousal_spontaneous", "arousal_plm", "", "arousal_bruxism", "arousal_noise"
        # "NasalSnore": "arousal_snore",
        # "CheyneStokesRespiration": "resp_cheynestokesbreath",
        "RERA|RERA": "arousal_rera",
        "Arousal|Arousal ()": "arousal",
        "ASDA arousal|Arousal (ASDA)": "arousal_asda",
        "Arousal resulting from Chin EMG|Arousal (CHESHIRE)": "arousal_chin",
        "Arousal|Arousal (STANDARD)": "arousal",
        # "LegMovement": "LegMovement",
    }

    eventMapApnea = {
        "Obstructive apnea|Obstructive Apnea": "resp_obstructiveapnea",
        "Central apnea|Central Apnea": "resp_centralapnea",
        "Hypopnea|Hypopnea": "resp_hypopnea",
        "Mixed apnea|Mixed Apnea": "resp_mixedapnea",
        # "SpO2 desaturation|SpO2 desaturation": "spo2_desaturation",
        # "SpO2 artifact|SpO2 artifact": "SpO2_artifact",
        # "Unsure|Unsure": "undefined",
    }

    eventMapSpO2 = {
        "SpO2 desaturation|SpO2 desaturation": "spo2_desaturation",
        "SpO2 artifact|SpO2 artifact": "SpO2_artifact",
    }

    # eventMapArtefacts = {
    #     "SpO2 artifact|SpO2 artifact": "SpO2_artifact",
    # }

    def getPath(self, xml, path):
        path = "./ScoredEvents/ScoredEvent" + path
        return xml.findall(path)

    def loadEvents(
        self,
        path,
        eventMap,
        durationChild="Duration",
        startChild="Start",
        typeChild="EventType",
        conceptChild="EventConcept",
        defaultState="ignore",
        minDuration=0,
        replaceName=None,
    ):
        tags = self.getPath(self.metaXML, path)
        if tags is None:
            raise AnnotationNotFound(path)

        events = []

        lastDefaultEvent = None
        for tag in tags:
            name = tag.find(conceptChild).text
            startValue = float(tag.find(startChild).text)
            if startValue is None:
                raise AnnotationInvalid(path + [startChild])

            startInSeconds = float(startValue)
            # if the name is in the eventMap it will be added to the annotations
            if name in eventMap:
                event = Event()
                event.start = startInSeconds
                event.manual = True
                eventName = replaceName(tag) if replaceName else eventMap[name]

                event.name = eventName

                if durationChild is not None:
                    # if there is a duration the event will be saved as as 2 events:
                    # startTime, "(eventName"
                    # endTime, "eventName)"
                    durationValue = float(tag.find(durationChild).text)
                    if durationValue is None:
                        raise AnnotationInvalid(path + [durationChild])

                    durationInSeconds = float(durationValue)
                    if durationInSeconds > minDuration:
                        event.duration = durationInSeconds
                        events.append(event)
                else:
                    # if its without a duration, it is considered a permanent state change
                    # that will persist until it is changed again
                    if lastDefaultEvent is not None:
                        lastDefaultEvent.duration = event.start - lastDefaultEvent.start
                    events.append(event)
                    lastDefaultEvent = event
            # else:
            #     self.logWarning("Event " + name + " not in EventMap.")

        if lastDefaultEvent is not None and self.lightOn is not None:
            lastDefaultEvent.duration = self.lightOn - lastDefaultEvent.start

        return events

    def loadAnnotation(self, xmlFile) -> List[Event]:
        self.loadXmlFile(xmlFile)

        allEvents = []
        allEvents += self.loadEvents(
            "[EventType='Stages|Stages']",
            self.stageMap,
        )
        allEvents += self.loadEvents(
            "[EventType='Arousals|Arousals']",
            self.eventMap,
        )

        allEvents += self.loadEvents(
            "[EventType='Respiratory|Respiratory']",
            self.eventMapApnea,
        )

        # allEvents += self.loadEvents(
        #     "[EventType='Respiratory|Respiratory']",
        #     self.eventMapSpO2,
        # )

        self.lightOff = 0
        self.lightOn = None

        return allEvents

    def fillRecord(self, record, xmlFile):
        pass



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pyPhasesRecordloaderSHHS",
    version="v0.1.3"[1:],
    author="Franz Ehrlich",
    author_email="fehrlichd@gmail.com",
    description="Adds a record loaders to the pyPhases package",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitlab.com/tud.ibmt.public/pyphases/pyphasesrecordloader/",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=["pyPhases", "pyPhasesRecordloader", "pyedflib"],
    python_requires=">=3.5",
)



from teleschlafmedizin.model.PSGSignal import PSGSignal
from teleschlafmedizin.model.Annotation import Annotation
from teleschlafmedizin.model.recordManager.RecordMeta import Channel, Record, RecordAnnotation, RecordChannel
from pyPhases.util.Logger import classLogger
from teleschlafmedizin.model.util.DynamicModule import DynamicModule
from . import recordManager as recordManagerPath


class ParseError(Exception):
    pass


class AnnotationException(Exception):
    path = []
    name = ""

    def __init__(self, path):
        self.path = path
        self.name = path[-1]
        super().__init__(self.getMessage())


class AnnotationNotFound(AnnotationException):
    def getMessage(self):
        return "Annotation was not found in the XML file: %s" % (self.path + [self.name])


class AnnotationInvalid(AnnotationException):
    def getMessage(self):
        return "Annotation is invalid: %s" % (self.path)


class ChannelsNotPresent(Exception):
    channels = []

    def __init__(self, channels, recordid="Unknown"):
        msg = "Channels of record %s where not present: %s" % (recordid, channels)
        super().__init__(msg)
        self.channels = channels


@classLogger
class RecordLoader:
    recordLoader = DynamicModule(recordManagerPath)

    def __init__(self, filePath="", targetFrequency=50, targetSignals=[], targetSignalTypes=[], optionalSignals=[]) -> None:
        self.filePath = filePath
        self.targetFrequency = targetFrequency
        self.optionalSignals = optionalSignals
        self.targetSignals = targetSignals
        self.targetSignalTypes = targetSignalTypes
        # lightOff and lightOn are in seconds !
        self.lightOff = 0
        self.lightOn = None
        self.classificationConfig = {}
        self.exportsEventArray = False
        self.firstSleep = None
        self.lastSleep = None
        self.signalTypeDict = dict(zip(self.targetSignals, self.targetSignalTypes))
        self.useDigitalSignals = False

    def delete(self, recordName):
        pass

    def exist(self, recordName):
        pass

    def loadRecord(self, recordName):
        pass

    def getSignal(self, recordName) -> PSGSignal:
        pass

    def loadAnnotation(self, recordName):
        pass

    def getEventList(self, recordName):
        pass

    def existAnnotation(self, recordId):
        """Check if an annotation exist for a given recordId.

        Returns:
            boolean: annotation exist
        """
        return False

    @staticmethod
    def get() -> "RecordLoader":
        return RecordLoader.recordLoader.get()

    def fillRecordFromPSGSignal(self, record: Record, psgSignal: PSGSignal):

        record.recordName = psgSignal.recordId

        for signal in psgSignal.signals:
            signal.typeStr = self.getSignalTypeStrFromDict(signal.name)
        psgSignal.checkPSGQuality()

        for signal in psgSignal.signals:
            channel = Channel()
            recordChannel = RecordChannel()

            channel.name = signal.name
            channel.dimension = signal.dimension
            channel.min = signal.physicalMin
            channel.max = signal.physicalMax
            recordChannel.transducer = signal.transducer
            recordChannel.frequency = signal.frequency
            recordChannel.prefilter = signal.prefilter
            recordChannel.quality = signal.quality

            recordChannel.Channel = channel
            record.recordChannels.append(recordChannel)
        return record

    def getSignalTypeStrFromDict(self, signalName):
        if self.signalTypeDict == {}:
            self.signalTypeDict = dict(zip(self.targetSignals, self.targetSignalTypes))
        if signalName in self.signalTypeDict:
            signalTypeStr = self.signalTypeDict[signalName]
        else:
            self.logError("Signal '%s' had no type when initilizing the RecordLoader" % str(signalName))
            signalTypeStr = "unknown"
        return signalTypeStr


class RecordWriter:
    recordWriter = DynamicModule(recordManagerPath)
    record: Record = None

    def writerRecord(recordName):
        pass

    def writeAnnotation(self, annotation: Annotation):
        pass

    def writeDataAnnotation(self, dataAnnotation: RecordAnnotation):
        """Writes a RecordAnnotationannotation to the Record

        Args:
            dataAnnotation (Annotation): RecordAnnotation with events and an Annotation with name

        """
        a = self.annotation.fromDataAnnotation(dataAnnotation)
        return self.writeAnnotation(a)

    @staticmethod
    def get() -> "RecordWriter":
        return RecordWriter.recordWriter.get()


class RecordManager:
    @staticmethod
    def getReader() -> RecordLoader:
        return RecordLoader.get()

    def getWriter() -> RecordWriter:
        return RecordWriter.get()
